(***************************************************************************
 *
 * Abstract Syntax Tree Interface
 *
 * Author: Gerald Baumgartner
 * Date:   18 October 2001
 * Date:   4 April 2010
 *
 ***************************************************************************)

signature AB_SYN =
  sig
    eqtype iSet
    eqtype iSetSeq
    eqtype expTree
    eqtype fusionTree

    datatype arithOp = Add | Sub | Mul | Div

    datatype abSyn = Null
		   | IntLit of IntInf.int
		   | Var of string
		   | Fun of string * string list
		   | Array of string * string list
		   | BinOp of arithOp * abSyn * abSyn
		   | Assign of abSyn * abSyn
		   | OpAssign of arithOp * abSyn * abSyn
		   | For of iSet * abSyn
		   | Seq of abSyn list
		   | Write of string * string list * abSyn
		   | Return of abSyn
		   | VarDecl of string * int list
		   | FunDecl of string * string * int list
		   | TempDecl of string * int list
		   | ResultDecl of string * int list
		   | Function of string * string * abSyn
		   | Alloc of string * int list
		   | Free of string * int list

    exception MalformedFusionTree of iSetSeq * (iSetSeq*IntInf.int*abSyn) list

    val buildSyntaxTree : (expTree option * fusionTree) -> abSyn
    val buildSyntaxTreeFile : (expTree option * fusionTree) list -> abSyn
    val toString : abSyn -> string
    val toC : abSyn -> string
  end
