(***************************************************************************
 *
 * Parse Functions
 *
 * Author: Gerald Baumgartner
 * Date:   20 July 2001
 *
 ***************************************************************************)

structure ExpLrVals = ExpLrValsFun (
  structure Token = LrParser.Token
  structure ExpTree = ExpTree);

structure ExpLex = ExpLex (structure Tokens = ExpLrVals.Tokens);

structure ExpParser = Join (
  structure LrParser = LrParser
  structure ParserData = ExpLrVals.ParserData
  structure Lex = ExpLex);


local
  fun error (errmsg, line, _) =
    print ("Parse error.  Line " ^ (Int.toString line) ^ ": " ^ errmsg ^ "\n")

  fun parse' file =
    let
      fun reader _ = TextIO.input file
      val lexer = ExpParser.makeLexer reader
      val (tree, _) = ExpParser.parse (15, lexer, error, ())
    in
      TextIO.closeIn file;
      tree
    end

in
  fun parseString string =
    parse' (TextIO.openString string)

  fun parse filename = 
    parse' (TextIO.openIn filename)
end
